from typing import List, Any

from dask.dataframe import DataFrame


def drop_rows_by_index(dd: DataFrame, drop_index: List[Any]):
    """
    API to remove rows specified by index labels.

    Args:
        df (dask.dataframe.DataFrame): target dask dataframe
        drop_index (List[Any]): index labels to drop from dataframe
    Returns:
        result dask dataframe
    Examples:
        >>> import dask.dataframe
        >>> import pandas
        >>> pd = pandas.DataFrame({'a': range(0, 60, 10), 'b': range(0, 600, 100)}, index=range(0, 6))
        >>> print(pd)
            a    b
        0   0    0
        1  10  100
        2  20  200
        3  30  300
        4  40  400
        5  50  500
        >>> dd = dask.dataframe.from_pandas(pd, npartitions=2)
        >>> result = drop_rows_by_index(dd, [0, 2, 3])
        >>> print(result.compute())
            a    b
        1  10  100
        4  40  400
        5  50  500
    """
    return dd.map_partitions(lambda pd: pd[~pd.index.isin(drop_index)])



from typing import Any, List

import pandas
from dask.dataframe import DataFrame
from framania.pandasmania.stack import stack_list_column as pandas_stack_list_column


def stack_list_column(dd: DataFrame, list_column: Any, output_dtype: Any, keep_columns: List[Any] = None):
    """
    API to create stack dask dataframe from column containing list-like objects.

    Args:
        df (dask.dataframe.DataFrame): target dask dataframe
        list_column (Any): name of column containing list-like objects.
        output_dtype (Any): dtype of output column. may depends on contents of list.
        keep_columns (List[Any]): result dataframe will contains original index, stacked column, and keep_columns
    Returns:
        result dask dataframe
    Examples:
        >>> import dask.dataframe
        >>> import pandas
        >>> pd = pandas.DataFrame({'a': ['1,2,3', '2,3,4', '3,4'], 'b': [1, 2, 3], 'idx': [0, 1, 2]}).set_index('idx')
        >>> pd['a-list'] = pd['a'].str.split(',')
        >>> print(pd)
        ... # doctest: +NORMALIZE_WHITESPACE
                 a  b     a-list
        idx
        0    1,2,3  1  [1, 2, 3]
        1    2,3,4  2  [2, 3, 4]
        2      3,4  3     [3, 4]
        >>> dd = dask.dataframe.from_pandas(pd, npartitions=2)
        >>> result = stack_list_column(dd, 'a-list', 'str', ['b'])
        >>> print(result.compute())
        ... # doctest: +NORMALIZE_WHITESPACE
             b a-list
        idx
        0    1      1
        0    1      2
        0    1      3
        1    2      2
        1    2      3
        1    2      4
        2    3      3
        2    3      4
    """
    meta = {k_column: dd[k_column].dtype for k_column in keep_columns}
    meta[list_column] = output_dtype

    return dd.map_partitions(lambda pd: pandas_stack_list_column(pd, list_column, output_dtype, keep_columns),
                             meta=meta)



from typing import Any, List

import pandas
from pandas import DataFrame


def stack_list_column(pd: DataFrame, list_column: Any, output_dtype: Any, keep_columns: List[Any]=None):
    """
    API to create stack pandas dataframe from column containing list-like objects.

    Args:
        df (pandas.DataFrame): target pandas dataframe
        list_column (Any): name of column containing list-like objects.
        output_dtype (Any): dtype of output column. may depends on contents of list.
        keep_columns (List[Any]): result dataframe will contains original index, stacked column, and keep_columns
    Returns:
        result pandas dataframe
    Examples:
        >>> import pandas
        >>> pd = pandas.DataFrame({'a': ['1,2,3', '2,3,4', '3,4'], 'b': [1, 2, 3], 'idx': [0, 1, 2]}).set_index('idx')
        >>> pd['a-list'] = pd['a'].str.split(',')
        >>> print(pd)
        ... # doctest: +NORMALIZE_WHITESPACE
                 a  b     a-list
        idx
        0    1,2,3  1  [1, 2, 3]
        1    2,3,4  2  [2, 3, 4]
        2      3,4  3     [3, 4]
        >>> result = stack_list_column(pd, 'a-list', 'str', ['b'])
        >>> print(result)
        ... # doctest: +NORMALIZE_WHITESPACE
             b a-list
        idx
        0    1      1
        0    1      2
        0    1      3
        1    2      2
        1    2      3
        1    2      4
        2    3      3
        2    3      4
    """

    ipd = pd.set_index(keep_columns, append=True)

    index = []
    values = []

    # TODO: more efficient stack implementation if exists
    for idx, row in ipd[list_column].iteritems():
        index.extend([idx] * len(row))
        values.extend(row)
    ix = pandas.Index(index, names=ipd.index.names)
    s = pandas.Series(values, dtype=output_dtype, index=ix)
    result = pandas.DataFrame({list_column: s})
    result.reset_index(keep_columns, drop=False, inplace=True)
    return result
